=begin
(c) TIG 2009

  drape_cpoints.rb

Adds Plugins Menu Item 'Drape Cpoints' which makes a regular grid of 
grouped cpoints over a 'mesh': it works on selected face(s), group(s) 
or component-instance(s) or combinations of these; other types of 
entities in the selection are ignored.
When the cpoint group is made you can use 'Triangulate Points' on the 
it re-make the 'mesh' with a regular grid.

v1.1 First release. 20090824

=end
require 'sketchup.rb'
class Drape_cpoints
  def Drape_cpoints::new()
    model=Sketchup.active_model
    ents=model.active_entities
    ss=model.selection
    ssa=ss.to_a
    ssa_classes=[];ssa.each{|e|ssa_classes<<e.class}
    ssa_classes.uniq!
    if not ssa[0] or not(ssa_classes.include?(Sketchup::Face)or ssa_classes.include?(Sketchup::Group)or ssa_classes.include?(Sketchup::ComponentInstance))
      UI.messagebox("First Select the 'Mesh'.\n\nIt must be Face(s) or Group(s) \nor Component-Instance(s)...")
      return nil
    end#if
    model.start_operation("Drape Cpoints")
    to_group=[]
    ssa.each{|e|
      if e.class==Sketchup::Face
        to_group<<e
        to_group<<e.edges
      else
        to_group<<e
      end#if
    }
    to_group.flatten!
    tgroup=ents.add_group(to_group)
    bb=tgroup.bounds
    maxX=bb.max.x;maxY=bb.max.y;maxZ=bb.max.z
    minX=bb.min.x;minY=bb.min.y;minZ=bb.min.z
    ss.clear
    ###
    prompts=["Grid Size [X=Y]: "]
    if not @step
      values=[20.feet]if model.options["UnitsOptions"]["LengthUnit"]<=1 ###imperial
      values=[5.m]if model.options["UnitsOptions"]["LengthUnit"]>1 ###metric
    else
      values=[@step]
    end#if
    results=inputbox(prompts,values,"Drape Cpoints")
    return nil if not results
    @step=results[0]
    ttgroup=ents.add_group()
    group=ents.add_group()
    ###populate group with cpoints
    numbX=((maxX-minX)/@step).abs.ceil
    numbY=((maxY-minY)/@step).abs.ceil
    numbX=numbX+1; numbY=numbY+1
    y=minY;z=maxZ+1
    tot=(numbX*numbY).to_s
    ct=1
    numbY.times do|v|
      x=minX
      numbX.times do|h|
        Sketchup::set_status_text("Making Cpoint "+(ct).to_s+" of "+tot)
        ct+=1
        ttgroup.entities.add_cpoint(Geom::Point3d.new(x,y,z))
        x=x+@step
      end#times
      y=y+@step
    end#times
    ### relocate cpoints
    ctr=1
    ttgroup.entities.each{|e|
      Sketchup::set_status_text("Draping Cpoint "+ctr.to_s+" of "+tot)
      ctr+=1
      hitMesh=false
      pt=e.position
     while not hitMesh
      ray=[pt,Geom::Vector3d.new(0,0,-1)]### down
      raytest=[]
      raytest=model.raytest(ray)
      if raytest ### hit something
        tp=raytest[0]
        if raytest[1][0]==tgroup ### hit ss
          group.entities.add_cpoint(tp)
          hitMesh=true
        else ### hit something else
          pt=tp
          hitMesh=false
        end#if
      else ### no hit so drop to ss min z
        group.entities.add_cpoint(Geom::Point3d.new(e.position.x,e.position.y,minZ))
        hitMesh=true
      end#if
     end#while
    }
    ttgroup.erase!
    tgroup.explode
    model.commit_operation
  end#def
end#class
### Menu ###-----------------------------------------------
if not file_loaded?(File.basename(__FILE__))
    UI.menu("Plugins").add_item("Drape Cpoints"){Drape_cpoints::new}
end#if
file_loaded(File.basename(__FILE__))